@@ -1,12 +1,12 @@
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
-from matplotlib.externals import six
+import six
 
 import os
 import re
 import signal
 import sys
-from matplotlib.externals.six import unichr
+from six import unichr
 
 import matplotlib
 
@@ -23,12 +23,10 @@ from matplotlib._pylab_helpers import Gcf
 from matplotlib.figure import Figure
 
 from matplotlib.widgets import SubplotTool
-try:
-    import matplotlib.backends.qt_editor.figureoptions as figureoptions
-except ImportError:
-    figureoptions = None
+import matplotlib.backends.qt_editor.figureoptions as figureoptions
 
-from .qt_compat import QtCore, QtGui, QtWidgets, _getSaveFileName, __version__
+from .qt_compat import (QtCore, QtGui, QtWidgets, _getSaveFileName,
+                        __version__, is_pyqt5)
 from matplotlib.backends.qt_editor.formsubplottool import UiSubplotTool
 
 backend_version = __version__
@@ -142,6 +140,9 @@ def _create_qApp():
         else:
             qApp = app
 
+    if is_pyqt5():
+        qApp.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps)
+
 
 class Show(ShowBase):
     def mainloop(self):
@@ -242,6 +243,18 @@ class FigureCanvasQT(QtWidgets.QWidget, FigureCanvasBase):
         w, h = self.get_width_height()
         self.resize(w, h)
 
+    @property
+    def _dpi_ratio(self):
+        # Not available on Qt4 or some older Qt5.
+        try:
+            return self.devicePixelRatio()
+        except AttributeError:
+            return 1
+
+    def get_width_height(self):
+        w, h = FigureCanvasBase.get_width_height(self)
+        return int(w / self._dpi_ratio), int(h / self._dpi_ratio)
+
     def enterEvent(self, event):
         FigureCanvasBase.enter_notify_event(self, guiEvent=event)
 
@@ -249,10 +262,14 @@ class FigureCanvasQT(QtWidgets.QWidget, FigureCanvasBase):
         QtWidgets.QApplication.restoreOverrideCursor()
         FigureCanvasBase.leave_notify_event(self, guiEvent=event)
 
+    def mouseEventCoords(self, pos):
+        x = pos.x() * self._dpi_ratio
+        # flip y so y=0 is bottom of canvas
+        y = self.figure.bbox.height - pos.y() * self._dpi_ratio
+        return x, y
+
     def mousePressEvent(self, event):
-        x = event.pos().x()
-        # flipy so y=0 is bottom of canvas
-        y = self.figure.bbox.height - event.pos().y()
+        x, y = self.mouseEventCoords(event.pos())
         button = self.buttond.get(event.button())
         if button is not None:
             FigureCanvasBase.button_press_event(self, x, y, button,
@@ -261,9 +278,7 @@ class FigureCanvasQT(QtWidgets.QWidget, FigureCanvasBase):
             print('button pressed:', event.button())
 
     def mouseDoubleClickEvent(self, event):
-        x = event.pos().x()
-        # flipy so y=0 is bottom of canvas
-        y = self.figure.bbox.height - event.pos().y()
+        x, y = self.mouseEventCoords(event.pos())
         button = self.buttond.get(event.button())
         if button is not None:
             FigureCanvasBase.button_press_event(self, x, y,
@@ -273,16 +288,12 @@ class FigureCanvasQT(QtWidgets.QWidget, FigureCanvasBase):
             print('button doubleclicked:', event.button())
 
     def mouseMoveEvent(self, event):
-        x = event.x()
-        # flipy so y=0 is bottom of canvas
-        y = self.figure.bbox.height - event.y()
+        x, y = self.mouseEventCoords(event)
         FigureCanvasBase.motion_notify_event(self, x, y, guiEvent=event)
         # if DEBUG: print('mouse move')
 
     def mouseReleaseEvent(self, event):
-        x = event.x()
-        # flipy so y=0 is bottom of canvas
-        y = self.figure.bbox.height - event.y()
+        x, y = self.mouseEventCoords(event)
         button = self.buttond.get(event.button())
         if button is not None:
             FigureCanvasBase.button_release_event(self, x, y, button,
@@ -291,9 +302,7 @@ class FigureCanvasQT(QtWidgets.QWidget, FigureCanvasBase):
             print('button released')
 
     def wheelEvent(self, event):
-        x = event.x()
-        # flipy so y=0 is bottom of canvas
-        y = self.figure.bbox.height - event.y()
+        x, y = self.mouseEventCoords(event)
         # from QWheelEvent::delta doc
         if event.pixelDelta().x() == 0 and event.pixelDelta().y() == 0:
             steps = event.angleDelta().y() / 120
@@ -323,15 +332,15 @@ class FigureCanvasQT(QtWidgets.QWidget, FigureCanvasBase):
             print('key release', key)
 
     def resizeEvent(self, event):
-        w = event.size().width()
-        h = event.size().height()
+        w = event.size().width() * self._dpi_ratio
+        h = event.size().height() * self._dpi_ratio
         if DEBUG:
             print('resize (%d x %d)' % (w, h))
             print("FigureCanvasQt.resizeEvent(%d, %d)" % (w, h))
         dpival = self.figure.dpi
         winch = w / dpival
         hinch = h / dpival
-        self.figure.set_size_inches(winch, hinch)
+        self.figure.set_size_inches(winch, hinch, forward=False)
         FigureCanvasBase.resize_event(self)
         self.draw_idle()
         QtWidgets.QWidget.resizeEvent(self, event)
@@ -490,6 +499,7 @@ class FigureManagerQT(FigureManagerBase):
             if self.toolbar is not None:
                 self.toolbar.update()
         self.canvas.figure.add_axobserver(notify_axes_change)
+        self.window.raise_()
 
     def full_screen_toggle(self):
         if self.window.isFullScreen():
@@ -562,6 +572,8 @@ class NavigationToolbar2QT(NavigationToolbar2, QtWidgets.QToolBar):
         NavigationToolbar2.__init__(self, canvas)
 
     def _icon(self, name):
+        if is_pyqt5():
+            name = name.replace('.png', '_large.png')
         return QtGui.QIcon(os.path.join(self.basedir, name))
 
     def _init_toolbar(self):
@@ -578,11 +590,10 @@ class NavigationToolbar2QT(NavigationToolbar2, QtWidgets.QToolBar):
                     a.setCheckable(True)
                 if tooltip_text is not None:
                     a.setToolTip(tooltip_text)
-
-        if figureoptions is not None:
-            a = self.addAction(self._icon("qt4_editor_options.png"),
-                               'Customize', self.edit_parameters)
-            a.setToolTip('Edit curves line and axes parameters')
+                if text == 'Subplots':
+                    a = self.addAction(self._icon("qt4_editor_options.png"),
+                                       'Customize', self.edit_parameters)
+                    a.setToolTip('Edit axis, curve and image parameters')
 
         self.buttons = {}
 
@@ -602,43 +613,47 @@ class NavigationToolbar2QT(NavigationToolbar2, QtWidgets.QToolBar):
         # reference holder for subplots_adjust window
         self.adj_window = None
 
-    if figureoptions is not None:
-        def edit_parameters(self):
-            allaxes = self.canvas.figure.get_axes()
-            if not allaxes:
-                QtWidgets.QMessageBox.warning(
-                    self.parent, "Error", "There are no axes to edit.")
-                return
-            if len(allaxes) == 1:
-                axes = allaxes[0]
-            else:
-                titles = []
-                for axes in allaxes:
-                    title = axes.get_title()
-                    ylabel = axes.get_ylabel()
-                    label = axes.get_label()
-                    if title:
-                        fmt = "%(title)s"
-                        if ylabel:
-                            fmt += ": %(ylabel)s"
-                        fmt += " (%(axes_repr)s)"
-                    elif ylabel:
-                        fmt = "%(axes_repr)s (%(ylabel)s)"
-                    elif label:
-                        fmt = "%(axes_repr)s (%(label)s)"
-                    else:
-                        fmt = "%(axes_repr)s"
-                    titles.append(fmt % dict(title=title,
-                                         ylabel=ylabel, label=label,
-                                         axes_repr=repr(axes)))
-                item, ok = QtWidgets.QInputDialog.getItem(
-                    self.parent, 'Customize', 'Select axes:', titles, 0, False)
-                if ok:
-                    axes = allaxes[titles.index(six.text_type(item))]
-                else:
-                    return
+        # Esthetic adjustments - we need to set these explicitly in PyQt5
+        # otherwise the layout looks different - but we don't want to set it if
+        # not using HiDPI icons otherwise they look worse than before.
+        if is_pyqt5():
+            self.setIconSize(QtCore.QSize(24, 24))
+            self.layout().setSpacing(12)
 
-            figureoptions.figure_edit(axes, self)
+    if is_pyqt5():
+        # For some reason, self.setMinimumHeight doesn't seem to carry over to
+        # the actual sizeHint, so override it instead in order to make the
+        # aesthetic adjustments noted above.
+        def sizeHint(self):
+            size = super(NavigationToolbar2QT, self).sizeHint()
+            size.setHeight(max(48, size.height()))
+            return size
+
+    def edit_parameters(self):
+        allaxes = self.canvas.figure.get_axes()
+        if not allaxes:
+            QtWidgets.QMessageBox.warning(
+                self.parent, "Error", "There are no axes to edit.")
+            return
+        if len(allaxes) == 1:
+            axes = allaxes[0]
+        else:
+            titles = []
+            for axes in allaxes:
+                name = (axes.get_title() or
+                        " - ".join(filter(None, [axes.get_xlabel(),
+                                                 axes.get_ylabel()])) or
+                        "<anonymous {} (id: {:#x})>".format(
+                            type(axes).__name__, id(axes)))
+                titles.append(name)
+            item, ok = QtWidgets.QInputDialog.getItem(
+                self.parent, 'Customize', 'Select axes:', titles, 0, False)
+            if ok:
+                axes = allaxes[titles.index(six.text_type(item))]
+            else:
+                return
+
+        figureoptions.figure_edit(axes, self)
 
     def _update_buttons_checked(self):
         # sync button checkstates to match active mode
